@@ -19,13 +19,12 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import os, sys, struct, operator
-import math, numarray,optparse, FFT
-import ossaudiodev, audioop
+import os, sys, struct
+import math, numarray, optparse, FFT
 
-__version__ = "$Revision: 1.3 $"
+__version__ = "$Revision: 1.6 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
-__depends__ = ['OSSAudioDev', 'FFT', 'Numeric-Extension', 'Python-2.4']
+__depends__ = ['FFT', 'Numeric-Extension', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
 This code is distributed under the terms of the GNU General Public License."""
 
@@ -46,14 +45,14 @@ MIN_RATE = 8000
 MIN_TONETIME = 0.1
 SUBWINDOW = 2
 
-DEF_DECODE_OVERPOWER = 30
-DEF_MIN_DIFF23_POWER = 15
+DEF_MIN_F1_POWER = 0.05
+DEF_DECODE_OVERPOWER = 20
+DEF_MIN_DIFF23_POWER = 10
 DEF_MAX_DIFF12_POWER = 20
 
 PEAK_UP = 1
-PEAK_DOWN = 2
+PEAK_DOWN = 1
 
-# Common audio formats from ossaudiodev: 
 # dictionary: string_format : bit_order, sample size, signed/unsigned
 AFMT_TO_DEF = { "S8": "=bS", "U8": "=BU", "S16_LE": "<hS", "U16_LE": "<HU", "S16_BE": ">hS", "U16_BE": ">HU"}
 	
@@ -104,7 +103,6 @@ class Generator:
 		format = base_format.upper()
 		try: sampleformatdef = AFMT_TO_DEF[format]
 		except: raise NameError, "sample audio format not supported: %s" %base_format
-		self.sampleformat = getattr(ossaudiodev, "AFMT_" + format)
 		self.samplebyteorder, self.samplectype, self.samplesign  = sampleformatdef
 		self.float_to_audio = (1 << len(struct.pack(self.samplectype, 0)) * 8) / 2.0
 		self.audio_offset = 0
@@ -176,6 +174,7 @@ class Decoder:
 		self.sensibility = args.get("sensibility", 1.0)
 		
 		# Decoding parameters
+		self.min_f1_power = DEF_MIN_F1_POWER * self.sensibility
 		self.decode_overpower = DEF_DECODE_OVERPOWER / self.sensibility
 		self.max_diff12_power = DEF_MAX_DIFF12_POWER * self.sensibility
 		self.min_diff23_power = DEF_MIN_DIFF23_POWER / self.sensibility
@@ -184,14 +183,13 @@ class Decoder:
 		format = base_format.upper()
 		try: sampleformatdef = AFMT_TO_DEF[format]
 		except: raise ValueError, "Audio format not supported: %s" %base_format
-		self.sampleformat = getattr(ossaudiodev, "AFMT_" + format)
 		self.samplebyteorder, self.samplectype, self.samplesign  = sampleformatdef
 		self.audio_to_float = 1.0 / float( (1 << len(struct.pack(self.samplectype, 0)) * 8) / 2.0 )
 		self.sample_length = len(struct.pack(self.samplectype, 0))
 		self.audio_offset = 0.0
 		if self.samplesign == "U": 
 			self.audio_offset = 1.0
-		self.input = numarray.array([])
+		self.input = []
 		d1 = [((f1, f2), freqs_to_key(f1, f2)) for f1 in DTMF_LOW_FREQS for f2 in DTMF_HIGH_FREQS]
 		d2 = [((f2, f1), freqs_to_key(f1, f2)) for f1 in DTMF_LOW_FREQS for f2 in DTMF_HIGH_FREQS]
 		self.freqs_to_key_dict = dict(d1 + d2)
@@ -227,6 +225,7 @@ class Decoder:
 		f1power, f1 = sorted_freqs[0]
 		f2power, f2 = sorted_freqs[1]
 		f3power, f3 = sorted_freqs[2]
+		f1realpower = (1000000 * f1power) / (self.windowsize)**2
 		try: key_max = self.freqs_to_key_dict[(f1,f2)]
 		except: key_max = None
 		self.debug("f1=%0.2f (%0.5f), f2=%0.2f (%0.5f), f3=%0.2f (%0.5f)" %(f1, f1power, f2, f2power, f3, f3power), 2)
@@ -239,15 +238,19 @@ class Decoder:
 			min_overpower = mean_power * self.decode_overpower
 
 			# Check if higher frequencies are greater than minimum acceptable power
-			if f2power > min_overpower and f1power < f2power * self.max_diff12_power and f2power > f3power * self.min_diff23_power:			
+			if f1realpower > self.min_f1_power and f2power > min_overpower and f1power < f2power * self.max_diff12_power and f2power > f3power * self.min_diff23_power:			
 				key_max = self.freqs_to_key_dict[(f1,f2)]
 				self.debug("key_max: %s. ** ok **" %key_max, 2)
+				self.debug("f1power -- %f" %(f1realpower), 1)
+				self.debug("f2power > min_overpower -- %f > %f" %(f2power, min_overpower), 1)
+				self.debug("f1power < f2power * max_diff12 -- %f < %f" %(f1power, f2power * self.max_diff12_power), 1)
+				self.debug("f2power > f3power * min_diff23 -- %f > %f" %(f2power, f3power * self.min_diff23_power), 1)
 			else: 
 				self.debug("key_max: %s. ko" %key_max, 2)
 				key_max = None
-			self.debug("f2power > min_overpower -- %f > %f" %(f2power, min_overpower), 2)
-			self.debug("f1power < f2power * max_diff12 -- %f < %f" %(f1power, f2power * self.max_diff12_power), 2)
-			self.debug("f2power > f3power * min_diff23 -- %f > %f" %(f2power, f3power * self.min_diff23_power), 2)
+				self.debug("f2power > min_overpower -- %f / %f" %(f2power, min_overpower), 2)
+				self.debug("f1power < f2power * max_diff12 -- %f / %f" %(f1power, f2power * self.max_diff12_power), 2)
+				self.debug("f2power > f3power * min_diff23 -- %f / %f" %(f2power, f3power * self.min_diff23_power), 2)
 
 		output = []
 
@@ -275,13 +278,15 @@ class Decoder:
 
 	################################################
 	def decode_buffer(self, buffer):
+		import audioop
 		if self.channels == 2:
 			buffer = audioop.tomono(buffer, self.sample_length, 0.5, 0.5)
 		format = self.samplebyteorder + str(len(buffer) / self.sample_length) + self.samplectype
 		abuffer = numarray.array(struct.unpack(format, buffer)) * self.audio_to_float
 		if self.audio_offset:
 			abuffer -= self.audio_offset
-		self.input = numarray.concatenate([self.input, abuffer]).tolist()
+		self.input = self.input + abuffer.tolist()
+		#self.input = numarray.concatenate([self.input, abuffer]).tolist()
 		dtmf_output = []
 		while len(self.input) >= self.windowsize:
 			window = numarray.array(self.input[:self.windowsize])
