@@ -19,17 +19,17 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import sys, os, optparse, pwd, grp
+import sys, os, optparse
 import time, select, popen2, re
-import audioop, signal, inspect
-import tempfile, shutil, syslog, errno
+import signal, inspect
+import syslog, errno
 
 # External phonepatch modules
 sys.path.append("/usr/lib/asterisk-phonepatch")
 import templateparser, dtmf, radio
-import radiocontrol, daemonize
+import radiocontrol
 
-__version__ = "$Revision: 1.8 $"
+__version__ = "$Revision: 1.10 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Asterisk', 'Sox', 'Festival', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -44,10 +44,13 @@ PL_CODES = {"XZ": 67.0, "WZ": 69.3, "XA": 71.9, "WA": 74.4, "XB": 77.0, "SP": 79
 	"7A": 192.8, "M1": 203.5, "8Z": 206.5, "M2" : 210.7, "M3": 218.1, "M4": 225.7,
 	"9Z": 229.2, "M5": 233.6, "M6": 241.8, "M7": 250.3, "0Z": 254.1}
 
-OUTCALL_IDNAME = "_phpoutcall_"
+VERSION = "0.5"
+OUTCALL_IDNAME = "_phonepatch_"
 GLOBAL_SECTION = "general"
 PIDFILE_DIR = "/var/run/asterisk"
 DEFAULT_SECTION = None
+ASTERISK_CTL = "/var/run/asterisk.ctl"
+
 
 ###############################
 ###############################
@@ -94,7 +97,8 @@ class Phonepatch:
 		self.phonepatch_phpconfig = None
 
 		# Asterisk definitions
-		self.asterisk_fifo = os.path.join("/tmp/php_fifo.raw")
+		self.pid = os.getpid()
+		self.asterisk_fifo = os.path.join("/tmp/php_fifo_%d.raw"%self.pid)
 		self.asterisk_samplerate = 8000
 		self.asterisk_channels = 1
 		
@@ -294,6 +298,7 @@ class Phonepatch:
 	def open_radio(self):
 		"""Open radio interface and configure PTT"""
 		
+		self.ctcss_decoder = self.getconf("ctcss_decoder_mintime")
 		self.ptt = self.carrier = self.radio_control = None
 		control = self.getconf("radio_control")
 		if control != "off" and (self.getconf("ptt") or self.getconf("carrier_detection") in ("on", "audio")):
@@ -332,7 +337,7 @@ class Phonepatch:
 		# Create radio instance (control soundcard and PTT)
 		try:self.radio = radio.Radio(self.getconf("soundcard_device"), self.asterisk_samplerate, \
 			self.ptt, self.carrier, verbose=self.modules_verbose, fullduplex = self.getconf("full_duplex"), \
-			soundcard_retries = 5, latency = self.getconf("soundcard_latency"), ctcss_decoder=self.getconf("ctcss_decoder_mintime"))
+			soundcard_retries = 5, latency = self.getconf("soundcard_latency"), ctcss_mintime=self.ctcss_decoder)
 		except IOError, detail:
 			self.debug("open_radio: %s" %str(detail), 1)
 			sys.exit(1)
@@ -429,7 +434,7 @@ class Phonepatch:
 		return audio_buffer
 
 	###################################
-	def play(self, play_radio = False, play_asterisk = False, args = None, max_time = None, test_function = None, loop = None):
+	def play(self, play_radio=False, play_asterisk=False, args=None, max_time=None, test_function=None, loop=None, flush_asterisk=True):
 		"""Play either audio files or text (using festival) and returns bytes written.
 		
 		play_radio/play_asterisk: Play the sound to the radio and/or asterisk link.
@@ -458,8 +463,7 @@ class Phonepatch:
 				except: pass
 				if option[0] == self.festival_indicator:
 					self.raw_buffer += self.play_text(option[len(self.festival_indicator):])
-				else:
-					self.raw_buffer += self.play_file(option)
+				else: self.raw_buffer += self.play_file(option)
 			# If something went wrong, raw_buffer will have no data
 			if not self.raw_buffer: 
 				self.debug("play: audio buffer is empty, giving up audio play")
@@ -486,7 +490,7 @@ class Phonepatch:
 				if not loop: break
 				buffer = self.raw_buffer
 			if test_function and not test_function(): 
-				return
+				written=None; break
 			if txdelay and time.time() < txtime:
 				continue
 			if play_radio and self.ptt and not pttflag: 
@@ -494,12 +498,12 @@ class Phonepatch:
 				pttflag = True
 			if play_radio:
 				try: self.radio.send_audio(buffer[:self.buffer_size], self.ctcss_tx)
-				except: self.debug("play: radio send_audio error"); return
+				except: self.debug("play: radio send_audio error"); written=None; break
 			if play_asterisk:
-				if not self.flush_asterisk(False) and not play_radio: break
-				try: self.secure_os(self.asterisk_in.write, buffer[:self.buffer_size]); self.secure_os(self.asterisk_in.flush)
-				except: self.debug("play: asterisk write error"); return
-			else: self.flush_asterisk()
+				if not self.flush_asterisk(False) and not play_radio: written=None; break
+				try: n = self.secure_os(self.asterisk_in.write, buffer[:self.buffer_size]); self.secure_os(self.asterisk_in.flush)
+				except: self.debug("play: asterisk write error"); written=None; break
+			elif flush_asterisk: self.flush_asterisk()
 
 			buffer = buffer[self.buffer_size:]
 			written += self.buffer_size
@@ -518,6 +522,7 @@ class Phonepatch:
 	#########################################
 	def set_gain(self, buffer, audio_gain):
 		"""Apply audio_gain to buffer"""
+		import audioop
 		if audio_gain == 1.0:
 			return buffer
 		return audioop.mul(buffer, self.sample_width, audio_gain)
@@ -536,7 +541,7 @@ class Phonepatch:
 		if not self.interface: return True
 		try: 
 			retsel = select.select([self.asterisk_out], [], [], 0.1)[0]
-			if self.asterisk_out not in retsel: return True
+			if self.asterisk_out not in retsel: return False
 			buffer = self.secure_os(os.read, self.asterisk_out, self.buffer_size)
 			if not buffer: return False
 			if write:
@@ -567,11 +572,12 @@ class Phonepatch:
 		break_reason = None
 		asterisk_timeout = 2.0
 		asterisk_time = time.time() + asterisk_timeout
-		try: self.empty_asterisk(0.1)
-		except: pass
-
+		input_fds = [self.asterisk_out, self.audio_fd]
+		try: self.empty_asterisk(0.05)
+		except: pass		
+		self.debug("audio_loop: start")
 		while 1:
-			try: retsel = select.select([self.asterisk_out, self.audio_fd], [], [])
+			try: retsel = select.select(input_fds, [], [])
 			except: self.debug("audio_loop: select error"); break
 			if not retsel: self.debug("audio_loop: select returned nothing"); break
 			now = time.time()
@@ -587,15 +593,15 @@ class Phonepatch:
 				if not buffer: self.debug("audio_loop: asterisk closed its read-descriptor"); break_reason = "asterisk"; break
 				buffer = self.set_gain(buffer, self.getconf("radio_audio_gain"))
 				self.radio.vox_toradio(buffer, self.ctcss_tx)
-				
+			
 			if self.audio_fd in retsel[0]:
 				# Radio -> Asterisk
-				buffer = self.radio.read_audio(self.buffer_size)
+				buffer = self.radio.read_audio(self.buffer_size, self.getconf("radio_audio_limit"))
 				if not buffer: self.debug("audio_loop: soundcard closed its descriptor"); break_reason = "radio"; break
 				buffer = self.set_gain(buffer, self.getconf("telephony_audio_gain"))
 				try: self.radio.vox_topeer(self.asterisk_in, buffer)
 				except: self.debug("audio_loop: asterisk closed its writing descriptor"); break_reason = "asterisk"; break
-								
+
 				# If hangup_button is configured, hangup line when received
 				if self.getconf("hangup_button"):
 					keys = self.dtmf_decoder.decode_buffer(buffer)
@@ -613,7 +619,7 @@ class Phonepatch:
 		
 		end_audio = self.getconf("end_audio")
 		if break_reason == "asterisk":
-			self.play(True, False, end_audio)
+			self.play(True, False, end_audio, flush_asterisk=False)
 		elif break_reason == "timeout":
 			self.play(True, True, end_audio)
 		elif break_reason == "user":
@@ -746,7 +752,7 @@ class Phonepatch:
 	def process_number(self, number):
 		# In CTCSS mode phonepatch extension is already set
 		if self.phonepatch_extension:
-			self.asterisk_extension = self.getconf("outcall_extension").replace("X", self.phonepatch_extension)
+			self.asterisk_extension = self.getconf("outcall_extension").replace("%x", self.phonepatch_extension)
 			return number
 		# DTMF mode
 		if not self.getconf("outcall_dtmf_extension_mode"):
@@ -754,23 +760,34 @@ class Phonepatch:
 			return number	
 		for phpext in self.phonepatch_extensions:				
 			mode = self.getconf("outcall_askfortone_mode", phpext)
-			dtmfid = self.getconf("outcall_dtmf_id", phpext).replace("X", phpext)
+			dtmfid = self.getconf("outcall_dtmf_id", phpext).replace("%x", phpext)
 			if mode == "dtmf" and dtmfid: 
 				if number[:len(dtmfid)] == dtmfid:
 					self.debug("process_number: phonepatch prefix match extension: %s" %phpext)
 					mode = self.getconf("outcall_askfortone_mode", phpext)
 					self.phonepatch_extension = phpext
 					number = number[len(dtmfid):]
-					self.asterisk_extension = self.getconf("outcall_extension").replace("X", phpext)
+					self.asterisk_extension = self.getconf("outcall_extension").replace("%x", phpext)
 					break
 		else: 
 			self.debug("process_number: outcall_dtmf_extension_mode enabled, but no outcall_dtmf_id matched the prefix number: %s" %number)
 			return
 		return number
 			
+	###################################
+	def check_asterisk_active(self):
+		try: rv = os.system("ps -C asterisk &>/dev/null") >> 8
+		except: return
+		return (rv == 0)
+		
 	###################################
 	def make_call(self, number, reopen=True):
 		"""Use outgoing calls Asterisk facility to call number"""
+		if not self.check_asterisk_active():
+			self.play(True, False, self.getconf("asterisk_inactive_audio"))
+			self.debug("make_call: asterisk not active")
+			return True
+
 		number = self.process_number(str(number))
 		if not number: 
 			self.debug("make_call: process_number() not succesful")
@@ -778,19 +795,21 @@ class Phonepatch:
 		if not self.getconf("outcall"):
 			self.debug("make_call: outcalls disabled for extension: %s" %self.phonepatch_extension)
 			return
-		
 		callerid = self.getconf("callerid")
 		if not callerid:
 			callerid = "%s <%s>" %(self.getconf("username"), self.phonepatch_extension)
 		self.debug("make_call: phonepatch: %s (%s) - destination: %s" %(self.phonepatch_extension, self.asterisk_extension, number))
-		channel = self.getconf("outcall_channel") + "/" + number
+		channel = self.getconf("outcall_channel").replace("%x", number)
+		
+		account = OUTCALL_IDNAME
 		
 		options = [("Channel", channel), ("MaxRetries", "0"), \
 			("RetryTime", "60"), ("Context", self.getconf("outcall_context")), \
 			("Extension", self.asterisk_extension), ("WaitTime", self.getconf("outcall_timeout")), \
-			("Priority", self.getconf("outcall_priority")), ("Account", OUTCALL_IDNAME), ("CallerID", callerid)]
-		
+			("Priority", self.getconf("outcall_priority")), ("Account", account), ("CallerID", callerid)]
+				
 		# Create a temporal file to write outgoing call options
+		import tempfile
 		tempfd, callpath = tempfile.mkstemp()
 
 		for key, value in options:
@@ -837,7 +856,6 @@ class Phonepatch:
 			else: continue
 			break
 			
-		# If the sleep_signal was not received, Asterisk was unable to connect to peer
 		if not self.sleep_signal:
 			self.debug("make_call: Asterisk was unable to connect")
 			return
@@ -897,10 +915,12 @@ class Phonepatch:
 
 	###################################
 	def get_asterisk_id(self):
+		import pwd
 		return pwd.getpwnam("asterisk")[2:4]
 
 	###################################
 	def init_daemon(self):
+		import grp
 		if self.background: 
 			syslog.openlog("phonepatch", syslog.LOG_PID, syslog.LOG_DAEMON)
 			self.modules_verbose = False
@@ -952,8 +972,9 @@ class Phonepatch:
 		while 1:
 			self.phonepatch_extension = None
 			if mode == "dtmf": self.debug("loop_daemon: waiting askfortone DTMF button: %s" %button)
-			self.debug("loop_daemon: CTCSS decoding enabled")
-			# CTCSS decoding always done (as it can be enabled inside an extension
+			if self.ctcss_decoder: self.debug("loop_daemon: CTCSS decoding enabled")
+			self.radio.clear_ctcss()
+			# CTCSS decoding always done (as it can be enabled inside an extension)
 			# DTMF decoding only if asked globally
 			while 1:
 				if self.sleep_signal:
@@ -988,8 +1009,12 @@ class Phonepatch:
 			
 			# AskForTone DTMF button received, now record the destination number
 			# TODO: Fullduplex
+			if not self.check_asterisk_active():
+				self.play(True, False, self.getconf("asterisk_inactive_audio"))
+				continue
+				
 			self.play(True, False, self.getconf("tone_audio"), max_time=self.getconf("tone_audio_time"), loop=True)
-			self.debug("loop_daemon: waiting for number and outcall_buttony")
+			self.debug("loop_daemon: waiting for number and outcall_button")
 			timeout_time = time.time() + self.getconf("tone_timeout")
 			dtmf_keys = []
 		
@@ -1036,23 +1061,30 @@ class Phonepatch:
 	###################################
 	def agicall(self, extension):
 		"""Phonepatch acting as AGI"""
-		self.set_state("agicall")
-		self.get_agi_keys(sys.stdin)
-		if extension not in self.configuration:
-			self.debug("agicall: phonepatch extension not defined: %s" %extension, 1)
-		self.phonepatch_extension = extension
-		phonepatch = self.getconf("phonepatch")
-		if not phonepatch:
-			if not self.phonepatch_configs:
-				self.debug("agicall: no phonepatchs defined", 1)
-			phonepatch = self.phonepatch_configs[0]
-			self.debug("agicall: no phonepatchs defined, selecting first: %s" %phonepatch)
-		if not self.init_php(phonepatch): 
-			self.debug("agicall: error in phonepatch init", 1)
-			return
-		if self.agi_keys.get("accountcode") == OUTCALL_IDNAME: 
-			self.outcall()
-		else: self.incall()
+		try:
+			self.set_state("agicall")
+			self.get_agi_keys(sys.stdin)
+			if extension not in self.configuration:
+				self.debug("agicall: phonepatch extension not defined: %s" %extension, 1)
+			self.phonepatch_extension = extension
+			phonepatch = self.getconf("phonepatch")
+			if not phonepatch:
+				if not self.phonepatch_configs:
+					self.debug("agicall: no phonepatchs defined", 1)
+				phonepatch = self.phonepatch_configs[0]
+				self.debug("agicall: no phonepatchs defined, selecting first: %s" %phonepatch)
+			if not self.init_php(phonepatch): 
+				self.debug("agicall: error in phonepatch init", 1)
+				return
+			if self.agi_keys.get("accountcode").find(OUTCALL_IDNAME) == 0: 
+				self.outcall()
+			else: self.incall()
+		except Exception, e:
+			# In case the phonepatch does not catch a signal
+			self.debug("agicall: exception not catched, here are the details:")
+			for line in str(e).splitlines():
+				self.debug(line)
+			self.signal_daemon(signal.SIGUSR2)
 		
 	###################################
 	def daemon(self, phonepatch, background=False, testcall=None):
@@ -1067,7 +1099,8 @@ class Phonepatch:
 			if not background:
 				self.debug("daemon: outcall_daemon disabled, daemon not loaded for phonepatch: %s" %phonepatch)
 			return
-		if self.background:	
+		if self.background:
+			import daemonize
 			pid = daemonize.daemonize(return_child=True)
 			if pid: return pid
 		self.set_state("daemon")
@@ -1081,7 +1114,13 @@ class Phonepatch:
 			self.delete_pidfile()
 			sys.exit(0)
 
-		self.loop_daemon()
+		while 1:
+			try: self.loop_daemon()
+			except Exception, e:
+				# In case the phonepatch does not catch a signal
+				self.debug("daemon: exception not catched, here are the details:")
+				for line in str(e).splitlines():
+					self.debug(line)
 
 	###################################
 	def outcall(self):
@@ -1169,6 +1208,7 @@ and spawn outgoing calls when required (radio DTMF controlled)"""
 		extname = args[0]
 		php = Phonepatch(configuration, verbose=options.verbose)
 		php.agicall(extname)
+
 	else: 
 		if options.phonepatch:
 			phonepatchs = [options.phonepatch]
