@@ -24,9 +24,9 @@ import errno, audioop
 import struct
 
 # External phonepatch modules
-import soundcard, ctcss
+import soundcard
 
-__version__ = "$Revision: 1.7 $"
+__version__ = "$Revision: 1.8 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -45,7 +45,7 @@ class Radio:
 
 	###############################
 	def __init__(self, soundcard_device, samplerate, ptt, carrier, verbose=False, \
-		soundcard_retries=1, fullduplex=False, latency=None, ctcss_decoder=None):
+		soundcard_retries=1, fullduplex=False, latency=None, ctcss_mintime=False):
 		"""Open a soundcard and PTT interface.
 
 		Use radio_control object to set PTT and get carrier-detection state.
@@ -87,9 +87,11 @@ class Radio:
 		self.set_carrier_state(False)
 	
 		# CTCSS generator/decoder		
-		if ctcss_decoder:
+		if ctcss_mintime:
+			import ctcss
 			self.ctcss_generator = ctcss.Generator(self.samplerate, self.sample_width)
-			self.ctcss_decoder = ctcss.Decoder(self.samplerate, self.sample_width, ctcss_decoder)
+			self.ctcss_decoder = ctcss.Decoder(self.samplerate, self.sample_width, ctcss_mintime)
+		else: self.ctcss_generator = self.ctcss_decoder = None
 		
 		# Open soundcard
 		self.soundcard = None
@@ -130,21 +132,37 @@ class Radio:
 		return self.soundcard
 
 	#####################################
-	def read_audio(self, size):
+	def limit_power(self, buffer, limit):
+		power = float(audioop.rms(buffer, self.sample_width)) / self.sample_max
+		if power > limit:
+			return audioop.mul(buffer, self.sample_width, limit/power)
+		else: return buffer
+		
+	#####################################
+	def read_audio(self, size, power_limit=1.0):
 		"""Read data from soundcard""" 
 		if not self.soundcard: self.debug("soundcard not opened"); return
 		buffer = self.soundcard.read(size)
 		if not buffer: return
 		buffer = self.update_carrier_state(buffer)
+		if power_limit < 1.0:
+			buffer = self.limit_power(buffer, power_limit)
 		return buffer
 
 	#####################################
 	def decode_ctcss(self, buffer):
+		if not self.ctcss_decoder: return
 		self.ctcss_decoder.decode_buffer(buffer)
+
+	#####################################
+	def clear_ctcss(self):
+		if not self.ctcss_decoder: return
+		self.ctcss_decoder.clear_tone()
 		
 	#####################################
 	def get_ctcss_tone(self):
-		#return 200.0 # test
+		#return 200.0
+		if not self.ctcss_decoder: return
 		return self.ctcss_decoder.get_tone()
 		
 	#####################################
@@ -273,7 +291,7 @@ class Radio:
 		if not self.soundcard: self.debug("soundcard not opened"); return
 		if not buffer: return
 				
-		if ctcss:
+		if ctcss and self.ctcss_generator:
 			freq, amplitude = ctcss
 			ctcss_buffer = self.ctcss_generator.generate(len(buffer), amplitude, freq)
 			buffer = audioop.add(buffer, ctcss_buffer, self.sample_width)
