@@ -18,15 +18,15 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import os, sys, re, array, errno
-import optparse, fcntl, math
+import os, array, errno
+import fcntl, math
 import ossaudiodev as oss
 
 ## TODO:
 
 ## Implement half/full-duplex mode
 
-__version__ = "$Revision: 1.3 $"
+__version__ = "$Revision: 1.5 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['OSSAudioDev', 'Python-2.3']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -60,7 +60,7 @@ class Soundcard:
 		try: self.sampleformat = getattr(oss, "AFMT_" + self.sampleformatstr.upper())
 		except: raise NameError, "Sample format not supported: %s" %self.sampleformatstr
 		self.fullduplex = bool(args.get("fullduplex", True))
-		self.fragmentsize = int(args.get("fragmentsize", 0))
+		self.fragmentsize = args.get("fragmentsize")
 		self.nonblock = bool(args.get("nonblock", False))
 		self.mode = str(args.get("mode", "rw"))
 		self.opened = False
@@ -94,7 +94,7 @@ class Soundcard:
 		if "w" not in self.mode:
 			raise IOError, "cannot write to %s, device not opened in write mode" %self.device
 		while 1:
-			try: self.fd.write(buffer)
+			try: n = self.fd.write(buffer)
 			except IOError, e:
 				if e.errno == errno.EINTR: continue
 				else: raise
@@ -106,6 +106,7 @@ class Soundcard:
 			raise IOError, "cannot open %s: device already opened" %self.device
 		self.opened = True
 		self.fd = oss.open(self.device, self.mode)
+		
 		# Set fragment size operation is not supported by ossaudiodev, do with ioctl
 		if self.fragmentsize:
 			arg = 0x7FFF0000 + int(math.log(self.fragmentsize, 2))
@@ -143,6 +144,7 @@ def show_options(sc):
 
 ###########################
 def main():
+	import sys, optparse
 	usage = """
 	soundcard.py [options] play | record
 	
