@@ -21,7 +21,7 @@
 # Standard Python modules
 import sys, optparse, re
 
-__version__ = "$Revision: 1.2 $"
+__version__ = "$Revision: 1.3 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Python-2.3']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -29,6 +29,7 @@ This code is distributed under the terms of the GNU General Public License."""
 
 COMMENT_CHARS = (";", "#")
 EQUAL_CHARS = ("=", ":")
+STATE_ON = ("on", "yes", "true", "1", "enabled", "enable", "active")
 
 ############################
 ############################
@@ -94,9 +95,8 @@ class Parser:
 			try: value = int(strvalue)
 			except: check = False
 		elif typedef == "state":
-			check &= strvalue in ("on", "off")
-			if strvalue == "on": value = True
-			else: value = False
+			check = True
+			value = (strvalue.lower() in STATE_ON)
 		elif typedef == "string":
 			check &= True
 			value = strvalue
